function T = TMCA_ENHANCEMENT(IMG)

MASKING = ones(size(IMG));
[r,c] = size(IMG);

%% Texture characteristic-based image decomposition using Morphological Component Analysis (MCA)

%Decompose by coarseness

dict1 = MakeList('SWT2','SWT2');
pars11 = MakeList('sym8','sym8');
pars12 = MakeList(3,3);
pars13 = MakeList(0,0);

figure;

part1 = MCA2_SIMPLE_COARENESS0(IMG.*MASKING,dict1,pars11,pars12,pars13,10,0,0,3);

P11 = part1(:,:,1);
P12 = part1(:,:,2);

%Normalize the result components
J11 = (P11-min(min(P11)))/(max(max(P11))-min(min(P11)));
J12 = (P12-min(min(P12)))/(max(max(P12))-min(min(P12)));

S1 = (J11+J12)/2;

%Decompose by contrast

dict2 = MakeList('SWT2','SWT2');
pars21 = MakeList('sym8','sym8');
pars22 = MakeList(3,3);
pars23 = MakeList(0,0);

figure;

part2 = MCA2_SIMPLE_CONTRAST0(IMG.*MASKING,dict2,pars21,pars22,pars23,5,0,0,3);

P21 = part2(:,:,1);
P22 = part2(:,:,2);

%Normalize the result components
J21 = (P21-min(min(P21)))/(max(max(P21))-min(min(P21)));
J22 = (P22-min(min(P22)))/(max(max(P22))-min(min(P22)));

S2 = (J21+J22)/2;

%Decompose by directionality

dict3 = MakeList('SWT2H','SWT2V');
pars31 = MakeList('Haar','Haar');
pars32 = MakeList(0,0);
pars33 = MakeList(0,0);

figure;

part3 = MCA2_SIMPLE(IMG.*MASKING,dict3,pars31,pars32,pars33,200,2,0,0,2);

P31 = part3(:,:,1);
P32 = part3(:,:,2);

%Normalize the result components
J31 = (P31-min(min(P31)))/(max(max(P31))-min(min(P31)));
J32 = (P32-min(min(P32)))/(max(max(P32))-min(min(P32)));

S3 = (J31+J32)/2;

%Decompose by line-likeness

dict4 = MakeList('SWT2','SWT2');
pars41 = MakeList('sym8','sym8');
pars42 = MakeList(3,3);
pars43 = MakeList(0,0);

figure;

part4 = MCA2_SIMPLE_LINELIKENESS(IMG.*MASKING,dict4,pars41,pars42,pars43,1,0,0,3);

P41 = part4(:,:,1);
P42 = part4(:,:,2);

J41 = (P41-min(min(P41)))/(max(max(P41))-min(min(P41)));
J42 = (P42-min(min(P42)))/(max(max(P42))-min(min(P42)));

S4 = J41+J42;

close all;
figure;
subplot(2,2,1);imshow(IMG);title('Input image');
subplot(2,2,2);imshow(S1);title('Sum of components');
subplot(2,2,3);imshow(J11);title('Coarse component');
subplot(2,2,4);imshow(J12);title('Fine component');
ax = axes('position',[0,0,1,1],'visible','off');
tx = text(0.35,0.05,'Decomposition by coarseness');
set(tx,'fontweight','bold');

figure;
subplot(2,2,1);imshow(IMG);title('Input image');
subplot(2,2,2);imshow(S2);title('Sum of components');
subplot(2,2,3);imshow(J21);title('High-contrast component');
subplot(2,2,4);imshow(J22);title('Low-contrast component');
ax = axes('position',[0,0,1,1],'visible','off');
tx = text(0.35,0.05,'Decomposition by contrast');
set(tx,'fontweight','bold');

figure;
subplot(2,2,1);imshow(IMG);title('Input image');
subplot(2,2,2);imshow(S3);title('Sum of components');
subplot(2,2,3);imshow(J31);title('Horizontal component');
subplot(2,2,4);imshow(J32);title('Vertical component');
ax = axes('position',[0,0,1,1],'visible','off');
tx = text(0.35,0.05,'Decomposition by directionality');
set(tx,'fontweight','bold');

figure;
subplot(2,2,1);imshow(IMG);title('Input image');
subplot(2,2,2);imshow(S4);title('Sum of components');
subplot(2,2,3);imshow(J41);title('Line-like component');
subplot(2,2,4);imshow(J42);title('Non-line-like component');
ax = axes('position',[0,0,1,1],'visible','off');
tx = text(0.35,0.05,'Decomposition by line-likeness');
set(tx,'fontweight','bold');


%% Image manipulation by manipulating every image components

fprintf('Manipulating the coarseness-based components...\n');

% Manipulate the coarseness-components

% Manipulation of coarse component
Options.kernelratio=4;
Options.windowratio=4;
Options.verbose=true;
Options.filterstrength=0.1;
T11 = NLMF_zhengguo(J11,Options);
% Manipulation of fine component
T12_sf = im2double(sf(im2uint8(J12),5,1));
T12 = T12_sf(1+2:r+2,1+2:c+2);

T1 = (T11+T12)/2;
% T1 = (T1-min(T1(:)))/(max(T1(:))-min(T1(:)));
alpha = 0.05;
T1(T1<0) = 0;
T1(T1>1) = 1;

% Manipulate the contrast-components

fprintf('Manipulating the contrast-based components...\n');

% Manipulation of high-contrast component
w = fspecial('laplacian',0);
y = imfilter(J21,w,'symmetric');
Y21 = medfilt2(y,[3 3],'symmetric');
T21 = J21-Y21;
% Manipulation of low-contrast component
t1 = 0.35;
t2 = 0.85;
T22 = zeros(size(J22));
for i = 1:numel(J22)
    if J22(i) < t1
        T22(i) = (J22(i)/t1).^2.*t1;
    elseif J22(i)>=t1 && J22(i)<t2
        T22(i) = ((J22(i)-t1)/(t2-t1)).^2.*(t2-t1)+t1;
    else
        T22(i) = ((J22(i)-t2)/(1-t2)).^0.5.*(1-t2) + t2;
    end
end

T2 = (T21+T22)/2;
% T2 = (T2-min(T2(:)))/(max(T2(:))-min(T2(:)));
beta = 0.05;
T2(T2<0) = 0;
T2(T2>1) = 1;

% Manipulate the directionality-components

fprintf('Manipulating the directionality-based components...\n');

% Manipulation of horizontal component

[J31_a,J31_h,J31_v,J31_d] = swt2(J31,1,'sym4');
J31_at = J31_a;
J31_ht = 1.5*J31_h;
J31_vt = zeros(size(J31_v));
J31_dt = J31_d;
T31 = iswt2(J31_at,J31_ht,J31_vt,J31_dt,'sym4');

% Manipulation of vertical component

[J32_a,J32_h,J32_v,J32_d] = swt2(J32,1,'sym4');
J32_at = J32_a;
J32_ht = zeros(size(J32_h));
J32_vt = 1.5*J32_v;
J32_dt = J32_d;
T32 = iswt2(J32_at,J32_ht,J32_vt,J32_dt,'sym4');

T3 = (T31+T32)/2;
% T3 = (T3-min(T3(:)))/(max(T3(:))-min(T3(:)));
T3(T3<0) = 0;
T3(T3>1) = 1;

% Manipulate the line-likeness-components

fprintf('Manipulating the line-likeness-based components...\n');

% Manipulation of line-like component

% T42 = imfilter(J42,fspecial('unsharp'),'symmetric');
T42 = adapthisteq(J42);

% Manipulation of non-line-like component

% J41_t = imadjust(J41,[0 1],[0 0.7]);
T41 = J41.^2;

% t11 = 0.35;
% t21 = 0.85;
% T41 = zeros(size(J41));
% for i = 1:numel(J41)
%     if J41(i) < t11
%         T41(i) = (J41(i)/t11).^2.*t11;
%     elseif J41(i)>=t11 && J41(i)<t21
%         T41(i) = ((J41(i)-t11)/(t21-t11)).^0.5.*(t21-t11)+t11;
%     else
%         T41(i) = ((J41(i)-t21)/(1-t21)).^2.*(1-t21) + t21;
%     end
% end


T4 = T41+T42;
% T4 = (T4-min(T4(:)))/(max(T4(:))-min(T4(:)));
gamma = 0.9;
T4(T4<0) = 0;
T4(T4>1) = 1;
%% Re-combine the manipulated components to generate the manipulated image

fprintf('Re-combining the manipulated components to get enhanced image...\n');
T = alpha*T1+beta*T2+gamma*T4;
T = (T-min(T(:)))/(max(T(:))-min(T(:)));
% T(T<0) = 0;
% T(T>1) = 1;