function [bs2,bs3,bs4,c2,c3,c4] = demo_segment(I)
[r,c] = size(I);
%Extract the texture features
fprintf('Extracting the local texture features of the image...\n');

d_m11 = MeasureofDirection(I,11);%larger better
d_m12 = MeasureofCoarseness(I,11);
d_m13 = MeasureofContrast(I,11);
d_m14 = MeasureofLinelikeness(I,11);

%K-means segmentation method. For different textural images, using
%different features as the clustering featuremap may increase the effect of
%segmentation.

fprintf('Segmenting the image by k-means clustering the texture feature vectors...\n');

d_m1 = [d_m12(:) d_m13(:)];
x1 = d_m1;
% opts = statset('Display','final');
[idx,ctrs] = kmeans(x1,2,'Distance','city','Replicates',5);
% [ctrs,idx] = vl_kmeans(x1,2);
bs2 = reshape(idx,r,c);
bs2 = bs2>1;

%Mean-shift segmentation method.For different textural images, using
%different features as the clustering featuremap may increase the effect of
%segmentation.

fprintf('Segmenting the image by mean-shift clustering the texture feature vectors...\n');

x2 = [d_m12(:) d_m13(:)]';
[clustCent,point2cluster,clustMembsCell] = MeanShiftCluster(x2,0.05);
bs3 = reshape(point2cluster,r,c);
bs3 = (bs3>1);

%GMM segmentation method.For different textural images, using
%different features as the clustering featuremap may increase the effect of
%segmentation.

fprintf('Segmenting the image by Gaussian mixture model clustering the texture feature vectors...\n');

d_m2 = [d_m12(:) d_m13(:)];
x3 = d_m2;
% gm = gmdistribution.fit(x3,4);
% opts = statset('Display','final','Regularize');
gm = fitgmdist(x3,2,'Start',idx,'Regularize',0.001);
% addpath(genpath('vlfeat-0.9.18'));
% gm = vl_gmm(x3,2);
point2cluster = cluster(gm,x3);
% rmpath(genpath('vlfeat-0.9.18'));

bs4 = reshape(point2cluster,r,c);
bs4 = bs4>1;

%Transfer the binary image to make the segmentation results comparable with
%the ground truth.

bs2 = transfer_binary(bs2);
bs3 = transfer_binary(bs3);
bs4 = transfer_binary(bs4);

%Calculate the accuracy of the segmentation results.

fprintf('Calculating the segmentation accuracy...\n');

%ground truth
g = zeros(size(bs2));
g(1:128,1:256) = 1;
%calculate the accuracy

a2 = (bs2==g);
a3 = (bs3==g);
a4 = (bs4==g);

c2 = sum(a2(:))/(r*c);
c3 = sum(a3(:))/(r*c);
c4 = sum(a4(:))/(r*c);

%preserve 4 decimal digits of accuracy

c2 = round(10^4*c2)/10^4;
c3 = round(10^4*c3)/10^4;
c4 = round(10^4*c4)/10^4;

%Transfer the binary segmentation result into gray-level to highlight the
%results.

bs2 = im2uint8(bs2);
bs3 = im2uint8(bs3);
bs4 = im2uint8(bs4);


bs2(bs2==255) = 210;
bs3(bs3==255) = 210;
bs4(bs4==255) = 210;

end

function T_S = transfer_binary(S)
    if S(64,128)==1
        T_S = S;
    else
        T_S = 1-S;    
    end    
end








